﻿;-TOP

; Comment : EventDesigner - Global Functions

; *******************************************************************


CompilerIf #PB_Compiler_OS = #PB_OS_Linux Or #PB_Compiler_OS = #PB_OS_MacOS

  Procedure UpdateWindow()
  
    Protected x, y, dx, dy, menu, status, toolbar
   
    menu = MenuHeight()
    status = StatusBarHeight(#Main_StatusBar)
    toolbar = ToolBarHeight(#Main_Toolbar)
    x = 0
    y = 0
    dx = WindowWidth(#Main)
    dy = WindowHeight(#Main) - menu - status - toolbar
    ResizeGadget(#Main_Splitter, x, y, dx, dy)
   
  EndProcedure

CompilerElse

  Procedure UpdateWindow()
  
    Protected x, y, dx, dy, menu, status, toolbar
   
    menu = MenuHeight()
    status = StatusBarHeight(#Main_StatusBar)
    toolbar = ToolBarHeight(#Main_Toolbar)
    x = 0
    y = toolbar
    dx = WindowWidth(#Main)
    dy = WindowHeight(#Main) - menu - status - toolbar
    ResizeGadget(#Main_Splitter, x, y, dx, dy)
   
  EndProcedure
  
CompilerEndIf

; ********

; Comment : Write Log Base
; Author  : mk-soft
; Create  : 27.08.2019
; Version : v1.01

; Enumeration CustemEvents #PB_Event_FirstCustomValue
;   #MyEventWriteLog
; EndEnumeration

; Declare MainWriteLog(Text.s)
; Declare Writelog(Text.s)

#WriteLogGadget = #Main_Listview_Status
#WriteLogMaxItems = 10000

Global flagWriteLogNoScroll = #False

Procedure MainWriteLog(Text.s) ; Call from Main Scope
  Protected cnt
  cnt = CountGadgetItems(#WriteLogGadget)
  If cnt >= #WriteLogMaxItems
    RemoveGadgetItem(#WriteLogGadget, 0)
    cnt -1
  EndIf
  AddGadgetItem(#WriteLogGadget, -1, Text)
  If Not flagWriteLogNoScroll
    SetGadgetState(#WriteLogGadget, cnt)
    SetGadgetState(#WriteLogGadget, -1)
  EndIf
EndProcedure

Procedure Writelog(Text.s) ; Call from Threads
  Protected *text.string
  *text = AllocateStructure(string)
  If *text
    *text\s = Text
    PostEvent(#MyEventWriteLog, 0, 0, 0, *text)
  EndIf
EndProcedure

Procedure DoEventWriteLog() ; Call from BindEvent
  Protected *text.string
  *text = EventData()
  If *text
    MainWriteLog(*text\s)
    FreeStructure(*text)
  EndIf
EndProcedure

BindEvent(#MyEventWriteLog, @DoEventWriteLog())

; ********

Procedure guiListCommonExistsFile(file.s)
  
  Protected temp.s, count, i
  
  count = CountGadgetItems(#Main_Listview_Common) - 1
  For i = 0 To count
    temp = GetGadgetItemText(#Main_Listview_Common, i)
    If temp = file
      ProcedureReturn #True
    EndIf
  Next
  
  ProcedureReturn #False
  
EndProcedure

; ----

Procedure guiListCommonDropFiles()
  
  Protected file.s, files.s, count, i
  
  files = EventDropFiles()
  count  = CountString(files, Chr(10)) + 1
  For i = 1 To count
    file = StringField(files, i, Chr(10))
    If Not guiListCommonExistsFile(file)
      AddGadgetItem(#Main_Listview_Common, -1, file)
      flagProjectDataSave = #False
    EndIf
  Next i
  
EndProcedure

; ----

Procedure guiListCommonAddFiles()
  
  Protected temp.s
  
  temp = OpenFileRequester("Form File", commonProjectData\LastPath, "Form|*.pb*|All files|*.*;*", 0, #PB_Requester_MultiSelection)
  If temp
    commonProjectData\LastPath = GetPathPart(temp)
  EndIf
  While temp
    If Not guiListCommonExistsFile(temp)
      AddGadgetItem(#Main_Listview_Common, -1, temp)
      flagProjectDataSave = #False
    EndIf
    temp = NextSelectedFileName()
  Wend
  
EndProcedure

; ----

Procedure guiListCommonDeleteFile()
  
  Protected index
  
  index = GetGadgetState(#Main_listview_Common)
  If index >= 0
    RemoveGadgetItem(#Main_Listview_Common, index)
    flagProjectDataSave = #False
  EndIf
  
EndProcedure

; ----

Procedure guiListCommonSetFirst()

  Protected index, temp.s
  
  index = GetGadgetState(#Main_listview_Common)
  If index >= 0
    temp = GetGadgetItemText(#Main_Listview_Common, index, 0)
    RemoveGadgetItem(#Main_Listview_Common, index)
    AddGadgetItem(#Main_Listview_Common, 0, temp)
    flagProjectDataSave = #False
  EndIf
  
EndProcedure

; ----

Procedure guiListCommonGetList()
  Protected rows, index
  
  ClearList(commonProjectData\Files())
  rows = CountGadgetItems(#Main_Listview_Common)
  If rows >= 0
    rows - 1
    For index = 0 To rows
      AddElement(commonProjectData\Files())
      commonProjectData\Files() = GetGadgetItemText(#Main_Listview_Common, index)
    Next
  EndIf
EndProcedure

; ----

Procedure guiAbout()
  MessageRequester("About", #Program + #LF$ + #LF$ + #Copyright, #PB_MessageRequester_Info)
EndProcedure

; ----

Procedure guiUserGadgets()
  OpenUserGadget()
  If IsWindow(#UserGadget)
    ForEach ValidUserGadgets()
      AddGadgetItem(#UserGadget_Listview_Gadget, -1, ValidUserGadgets())
    Next
  EndIf
EndProcedure

; ****

Procedure LoadValidUserGadget()
  Protected filename.s, xml
  
  ClearList(ValidUserGadgets())
  filename = commonProgramDataPath + "VaildUserGadgets.xml"
  If FileSize(filename) > 0
    xml = LoadXML(#PB_Any, filename)
    If xml And XMLStatus(xml) = #PB_XML_Success
      ExtractXMLList(MainXMLNode(xml), ValidUserGadgets(), #PB_XML_NoCase)
      FreeXML(xml)
    EndIf
  EndIf
  
EndProcedure

; ----

Procedure SaveVaildUserGadgets()
  Protected filename.s, xml
  
  filename = commonProgramDataPath + "VaildUserGadgets.xml"
  If ListSize(ValidUserGadgets())
    xml = CreateXML(#PB_Any)
    If xml
      If InsertXMLList(RootXMLNode(xml), ValidUserGadgets())
        FormatXML(xml, #PB_XML_ReFormat)
        SaveXML(xml, filename)
      EndIf
    EndIf
  Else
    If FileSize(filename) >= 0
      DeleteFile(filename)
    EndIf
  EndIf
  
EndProcedure

; ----

Procedure LoadProgramData()
  Protected filename.s, xml
  
  filename = commonProgramDataPath + "ProgramData.xml"
  If FileSize(filename) > 0
    xml = LoadXML(#PB_Any, filename)
    If xml And XMLStatus(xml) = #PB_XML_Success
      ExtractXMLStructure(MainXMLNode(xml), @commonProgramData, udtProgramData, #PB_XML_NoCase)
      FreeXML(xml)
    EndIf
  EndIf
  
EndProcedure

; ----

Procedure SaveProgramData()
  Protected filename.s, xml
  
  filename = commonProgramDataPath + "ProgramData.xml"
  xml = CreateXML(#PB_Any)
  If xml
    If InsertXMLStructure(RootXMLNode(xml), @commonProgramData, udtProgramData)
      FormatXML(xml, #PB_XML_ReFormat)
      SaveXML(xml, filename)
    EndIf
  EndIf

EndProcedure

; ----

Procedure LoadProjectData(LoadDefault = #True)
  Protected filename.s, xml
  
  If LoadDefault
    filename = commonProgramData\ProjectFile
  Else
    filename = OpenFileRequester("Load Project", "", "Project|*.xml;*.gui|All files (*.*)|*.*;*", 1)
  EndIf
  
  If filename = ""
    ProcedureReturn 0
  EndIf
  
  commonProgramData\ProjectPath = GetPathPart(filename)
  commonProgramData\ProjectFile = filename
  flagProjectDataSave = #True
  
  ClearStructure(@commonProjectData, udtProjectData)
  InitializeStructure(@commonProjectData, udtProjectData)
  
  ClearGadgetItems(#Main_Listview_Common)
  
  If FileSize(filename) > 0
    xml = LoadXML(#PB_Any, filename)
    If xml And XMLStatus(xml) = #PB_XML_Success
      ExtractXMLStructure(MainXMLNode(xml), @commonProjectData, udtProjectData, #PB_XML_NoCase)
      FreeXML(xml)
    EndIf
  EndIf
  
  ForEach commonProjectData\Files()
    AddGadgetItem(#Main_Listview_Common, -1, commonProjectData\Files())
  Next
  
  
  
EndProcedure

; ----

Procedure SaveProjectData(SaveDefault = #True)
  Protected filename.s, xml
  
  If SaveDefault
    filename = commonProgramData\ProjectFile
  EndIf
  
  If commonProgramData\ProjectPath = ""
    If FirstElement(commonProjectData\Files())
      commonProgramData\ProjectPath = GetPathPart(commonProjectData\Files())
    EndIf
  EndIf
  
  If filename = ""
    filename = SaveFileRequester("Save Project", commonProgramData\ProjectPath + "Project.xml", "Project|*.xml;*.gui", 0)
  EndIf
  
  If filename = ""
    ProcedureReturn #False
  EndIf
  
  commonProgramData\ProjectPath = GetPathPart(filename)
  commonProgramData\ProjectFile = filename
  
  xml = CreateXML(#PB_Any)
  If xml
    If InsertXMLStructure(RootXMLNode(xml), @commonProjectData, udtProjectData)
      FormatXML(xml, #PB_XML_ReFormat)
      SaveXML(xml, filename)
      flagProjectDataSave = #True
    EndIf
  EndIf
  
  ProcedureReturn #True
  
EndProcedure

; ----

Procedure InitProgram()
  
  commonProgramDataPath = GetUserDirectory(#PB_Directory_ProgramData)
  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
    commonProgramDataPath + "EventDesignerV3\"
  CompilerElse
    commonProgramDataPath + "EventDesignerV3/"
  CompilerEndIf
  If FileSize(commonProgramDataPath) <> 2
    CreateDirectory(commonProgramDataPath)
  EndIf
  
  LoadProgramData()
  LoadDataSection()
  LoadValidUserGadget()
  
  LoadProjectData(#True)
  
  flagProjectDataSave = #True
  
  StatusBarText(#Main_StatusBar, 0, "Project: " + commonProgramData\ProjectFile)
  
EndProcedure

; ----

Procedure ExitProgram()
  
  If Not flagProjectDataSave
    guiListCommonGetList()
    SaveProjectData(#True)
  EndIf
  SaveProgramData()
  
EndProcedure

; ----
; IDE Options = PureBasic 5.71 LTS (MacOS X - x64)
; CursorPosition = 379
; FirstLine = 367
; Folding = ----
; EnableXP